@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2006 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <inttypes.h>
 #include <unistd.h>
 #include <signal.h>
@@ -20,7 +7,7 @@
 #include "libcman.h"
 #include "cman_tool.h"
 
-#define OPTION_STRING		("m:n:v:e:2p:c:r:i:N:t:o:k:Vwfqah?d::")
+#define OPTION_STRING		("m:n:v:e:2p:c:r:i:N:t:o:k:F:Vwfqah?Xd::")
 #define OP_JOIN			1
 #define OP_LEAVE		2
 #define OP_EXPECTED		3
@@ -31,13 +18,14 @@
 #define OP_STATUS		8
 #define OP_NODES		9
 #define OP_SERVICES		10
+#define OP_DEBUG		11
 
 
 static void print_usage(int subcmd)
 {
 	printf("Usage:\n");
 	printf("\n");
-	printf("%s <join|leave|kill|expected|votes|version|wait|status|nodes|services> [options]\n",
+	printf("%s <join|leave|kill|expected|votes|version|wait|status|nodes|services|debug> [options]\n",
 	       prog_name);
 	printf("\n");
 	printf("Options:\n");
@@ -62,6 +50,7 @@ static void print_usage(int subcmd)
 		printf("  -q               Wait until the cluster is quorate\n");
 		printf("  -t               Maximum time (in seconds) to wait\n");
 		printf("  -k <file>        Private key file for AIS communications\n");
+		printf("  -X               Don't use CCS for configuration\n");
 		printf("\n");
 	}
 
@@ -109,6 +98,8 @@ static void print_usage(int subcmd)
 		printf("nodes              Show local record of cluster nodes\n");
 		printf("  -f                 Also show when node was last fenced\n");
 		printf("  -a                 Also show node address(es)\n");
+		printf("  -n <nodename>      Only show information for specific node\n");
+		printf("  -F <format>        Specify output format (see man page)\n");
 		printf("\n");
 	}
 
@@ -197,8 +188,10 @@ static void show_status(void)
 	char info_buf[PIPE_BUF];
 	char tmpbuf[1024];
 	cman_extra_info_t *einfo = (cman_extra_info_t *)info_buf;
+	cman_qdev_info_t qinfo;
 	int quorate;
-	int i,j;
+	int i;
+	int j;
 	int portnum;
 	char *addrptr;
 
@@ -223,7 +216,10 @@ static void show_status(void)
 							  einfo->ei_node_state));
 	printf("Nodes: %d\n", einfo->ei_members);
 	printf("Expected votes: %d\n", einfo->ei_expected_votes);
+	if (cman_get_quorum_device(h, &qinfo) == 0 && qinfo.qi_state == 2)
+		printf("Quorum device votes: %d\n", qinfo.qi_votes);
 	printf("Total votes: %d\n", einfo->ei_total_votes);
+
 	printf("Quorum: %d %s\n", einfo->ei_quorum, quorate?" ":"Activity blocked");
 	printf("Active subsystems: %d\n", cman_get_subsys_count(h));
 	printf("Flags:");
@@ -235,6 +231,8 @@ static void show_status(void)
 		printf(" Error");
 	if (einfo->ei_flags & CMAN_EXTRA_FLAG_DISALLOWED)
 		printf(" DisallowedNodes");
+	if (einfo->ei_flags & CMAN_EXTRA_FLAG_DIRTY)
+		printf(" Dirty");
 	printf(" \n");
 
 	printf("Ports Bound: ");
@@ -287,7 +285,7 @@ static void show_status(void)
 			printf("\n");
 		}
 	}
-
+	cman_finish(h);
 }
 
 static int node_compare(const void *va, const void *vb)
@@ -297,18 +295,157 @@ static int node_compare(const void *va, const void *vb)
 	return a->cn_nodeid - b->cn_nodeid;
 }
 
+static int node_filter(commandline_t *comline, const char *node)
+{
+	int i;
+
+	for (i = 0; i < comline->num_nodenames; i++) {
+		if (strcmp(comline->nodenames[i], node) == 0) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+static int get_format_opt(const char *opt)
+{
+	if (!opt)
+		return FMT_NONE;
+
+	if (!strcmp(opt, "id"))
+		return FMT_ID;
+	if (!strcmp(opt, "name"))
+		return FMT_NAME;
+	if (!strcmp(opt, "type"))
+		return FMT_TYPE;
+	if (!strcmp(opt, "addr"))
+		return FMT_ADDR;
+
+	return FMT_NONE;
+}
+
+
+static void print_node(commandline_t *comline, cman_handle_t h, int *format, struct cman_node *node)
+{
+	char member_type;
+	struct tm *ftime;
+	struct tm *jtime;
+	char jstring[1024];
+	int i,j,k;
+
+	if (comline->num_nodenames > 0) {
+		if (node_filter(comline, node->cn_name) == 0) {
+			return;
+		}
+	}
+
+	switch (node->cn_member) {
+	case 0:
+		member_type = 'X';
+		break;
+	case 1:
+		member_type = 'M';
+		break;
+	case 2:
+		member_type = 'd';
+		break;
+	default:
+		member_type = '?';
+		break;
+	}
+
+	jtime = localtime(&node->cn_jointime.tv_sec);
+	if (node->cn_jointime.tv_sec && node->cn_member)
+		strftime(jstring, sizeof(jstring), "%F %H:%M:%S", jtime);
+	else
+		strcpy(jstring, "                   ");
+
+	if (!comline->format_opts) {
+		printf("%4d   %c  %5d   %s  %s\n",
+		       node->cn_nodeid, member_type,
+		       node->cn_incarnation, jstring, node->cn_name);
+	}
+
+	if (comline->fence_opt && !comline->format_opts) {
+		char agent[255];
+		uint64_t fence_time;
+		int fenced;
+
+		if (!cman_get_fenceinfo(h, node->cn_nodeid, &fence_time, &fenced, agent)) {
+			if (fence_time) {
+				time_t fence_time_t = (time_t)fence_time;
+				ftime = localtime(&fence_time_t);
+				strftime(jstring, sizeof(jstring), "%F %H:%M:%S", ftime);
+				printf("       Last fenced:   %-15s by %s\n", jstring, agent);
+			}
+			if (!node->cn_member && node->cn_incarnation && !fenced) {
+				printf("       Node has not been fenced since it went down\n");
+			}
+		}
+	}
+
+	int numaddrs;
+	struct cman_node_address addrs[MAX_INTERFACES];
+
+	if (comline->addresses_opt || comline->format_opts) {
+		if (!cman_get_node_addrs(h, node->cn_nodeid, MAX_INTERFACES, &numaddrs, addrs) &&
+		    numaddrs)
+		{
+			if (!comline->format_opts) {
+				printf("       Addresses: ");
+				for (i = 0; i < numaddrs; i++)
+				{
+					print_address(addrs[i].cna_address);
+					printf(" ");
+				}
+				printf("\n");
+			}
+		}
+	}
+
+	if (comline->format_opts) {
+		for (j = 0; j < MAX_FORMAT_OPTS; j++) {
+			switch (format[j]) {
+			case FMT_NONE:
+				break;
+			case FMT_ID:
+				printf("%d ", node->cn_nodeid);
+				break;
+			case FMT_NAME:
+				printf("%s ", node->cn_name);
+				break;
+			case FMT_TYPE:
+				printf("%c ", member_type);
+				break;
+			case FMT_ADDR:
+				for (k = 0; k < numaddrs; k++) {
+					print_address(addrs[k].cna_address);
+					if (k != (numaddrs - 1)) {
+						printf(",");
+					}
+				}
+				printf(" ");
+				break;
+			default:
+				break;
+			}
+		}
+		printf("\n");
+	}
+}
+
 static void show_nodes(commandline_t *comline)
 {
 	cman_handle_t h;
 	int count;
 	int i;
+	int j;
 	int numnodes;
 	int dis_count;
+	int format[MAX_FORMAT_OPTS];
 	cman_node_t *dis_nodes;
 	cman_node_t *nodes;
-	struct tm *jtime;
-	struct tm *ftime;
-	char jstring[1024];
 
 	h = open_cman_handle(0);
 
@@ -322,6 +459,16 @@ static void show_nodes(commandline_t *comline)
 	if (!nodes)
 		die("cannot allocate memory for nodes list\n");
 
+	if (comline->format_opts != NULL) {
+		char *format_str = comline->format_opts;
+		char *format_tmp;
+		for (i = 0; i < MAX_FORMAT_OPTS; i++) {
+			format_tmp = strtok(format_str, ",");
+			format_str = NULL;
+			format[i] = get_format_opt(format_tmp);
+		}
+	}
+
 	if (cman_get_nodes(h, count, &numnodes, nodes) < 0)
 		die("cman_get_nodes failed: %s", cman_error(errno));
 
@@ -330,9 +477,8 @@ static void show_nodes(commandline_t *comline)
 	dis_nodes = malloc(sizeof(cman_node_t) * count);
 
 	if (cman_get_disallowed_nodes(h, count, &dis_count, dis_nodes) == 0) {
-		int i,j;
-		for (i=0; i<numnodes; i++) {
-			for (j=0; j<dis_count; j++) {
+		for (i = 0; i < numnodes; i++) {
+			for (j = 0; j < dis_count; j++) {
 				if (dis_nodes[j].cn_nodeid == nodes[i].cn_nodeid)
 					nodes[i].cn_member = 2;
 			}
@@ -347,77 +493,23 @@ static void show_nodes(commandline_t *comline)
 		printf("      members list may seem inconsistent across the cluster\n");
 	}
 
-	printf("Node  Sts   Inc   Joined               Name\n");
-	for (i=0; i<numnodes; i++) {
-		char member_type;
-
-		switch (nodes[i].cn_member) {
-		case 0:
-			member_type = 'X';
-			break;
-		case 1:
-			member_type = 'M';
-			break;
-		case 2:
-			member_type = 'd';
-			break;
-		default:
-			member_type = '?';
-			break;
-		}
-
-		jtime = localtime(&nodes[i].cn_jointime.tv_sec);
-		if (nodes[i].cn_jointime.tv_sec && nodes[i].cn_member)
-			strftime(jstring, sizeof(jstring), "%F %H:%M:%S", jtime);
-		else
-			strcpy(jstring, "                   ");
-
-		printf("%4d   %c  %5d   %s  %s\n",
-		       nodes[i].cn_nodeid, member_type,
-		       nodes[i].cn_incarnation, jstring, nodes[i].cn_name);
-
-		if (comline->fence_opt) {
-			char agent[255];
-			uint64_t fence_time;
-			int fenced;
-
-			if (!cman_get_fenceinfo(h, nodes[i].cn_nodeid, &fence_time, &fenced, agent)) {
-				if (fence_time) {
-					time_t fence_time_t = (time_t)fence_time;
-					ftime = localtime(&fence_time_t);
-					strftime(jstring, sizeof(jstring), "%F %H:%M:%S", ftime);
-					printf("       Last fenced:   %-15s by %s\n", jstring, agent);
-				}
-				if (!nodes[i].cn_member && nodes[i].cn_incarnation && !fenced) {
-					printf("       Node has not been fenced since it went down\n");
-				}
-			}
-		}
-
-		if (comline->addresses_opt)
-		{
-			int numaddrs;
-			struct cman_node_address addrs[MAX_INTERFACES];
-			if (!cman_get_node_addrs(h, nodes[i].cn_nodeid, MAX_INTERFACES, &numaddrs, addrs) &&
-				numaddrs)
-			{
-				int i;
-				printf("       Addresses: ");
-				for (i=0; i<numaddrs; i++)
-				{
-					print_address(addrs[i].cna_address);
-					printf(" ");
-				}
-				printf("\n");
-			}
-		}
+	if (!comline->format_opts) {
+		printf("Node  Sts   Inc   Joined               Name\n");
 	}
+
+	/* Print nodes */
+	for (i = 0; i < numnodes; i++) {
+		print_node(comline, h, format, &nodes[i]);
+	}
+
 	free(nodes);
+	free(dis_nodes);
+	cman_finish(h);
 }
 
 static void show_services(void)
 {
-	system("group_tool");
+	system("group_tool ls");
 }
 
 
@@ -587,6 +679,18 @@ static void kill_node(commandline_t *comline)
 	cman_finish(h);
 }
 
+static void set_debuglog(commandline_t *comline)
+{
+	cman_handle_t h;
+
+	h = open_cman_handle(1);
+
+	if (cman_set_debuglog(h, comline->verbose))
+		perror("setting debuglog failed");
+
+	cman_finish(h);
+}
+
 
 static int get_int_arg(char argopt, char *arg)
 {
@@ -684,9 +788,13 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 			comline->clustername_opt = TRUE;
 			break;
 
+		case 'F':
+			comline->format_opts = strdup(optarg);
+			break;
+
 		case 'V':
 			printf("cman_tool %s (built %s %s)\n",
-				CMAN_RELEASE_NAME, __DATE__, __TIME__);
+				RELEASE_VERSION, __DATE__, __TIME__);
 			printf("%s\n", REDHAT_COPYRIGHT);
 			exit(EXIT_SUCCESS);
 			break;
@@ -724,6 +832,9 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 			cont = FALSE;
 			break;
 
+		case 'X':
+			comline->noccs_opt = TRUE;
+			break;
 		default:
 			die("unknown option: %c", optchar);
 			break;
@@ -771,6 +882,10 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 			if (comline->operation)
 				die("can't specify two operations");
 			comline->operation = OP_SERVICES;
+		} else if (strcmp(argv[optind], "debug") == 0) {
+			if (comline->operation)
+				die("can't specify two operations");
+			comline->operation = OP_DEBUG;
 		} else if (strcmp(argv[optind], "remove") == 0) {
 			comline->remove = TRUE;
 		} else if (strcmp(argv[optind], "force") == 0) {
@@ -885,6 +1000,10 @@ int main(int argc, char *argv[])
 	case OP_SERVICES:
 		show_services();
 		break;
+
+	case OP_DEBUG:
+		set_debuglog(&comline);
+		break;
 	}
 
 	exit(EXIT_SUCCESS);
