@@ -104,7 +104,9 @@ tcp_exchange(int fd, fence_auth_type_t auth, void *key,
 		return -1;
 
 	/* Read return code */
-	read(fd, &ret, 1);
+	if (read(fd, &ret, 1) < 0)
+		return -1;
+
 	close(fd);
 	if (ret == 0)
 		printf("Remote: Operation was successful\n");
