@@ -1,24 +1,3 @@
-/*
-  Copyright Red Hat, Inc. 2003, 2004, 2006
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
-/*
- * Author: Lon Hohberger <lhh at redhat.com>
- */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,6 +20,7 @@
 
 /* Local includes */
 #include "mcast.h"
+#include "debug.h"
 
 /** 
   Sets up a multicast receive socket
@@ -62,7 +42,7 @@ ipv4_recv_sk(char *addr, int port)
 	/********************************
 	 * SET UP MULTICAST RECV SOCKET *
 	 ********************************/
-	dprintf(4, "Setting up ipv4 multicast receive (%s:%d)\n", addr, port);
+	dbg_printf(4, "Setting up ipv4 multicast receive (%s:%d)\n", addr, port);
 	sock = socket(PF_INET, SOCK_DGRAM, 0);
 	if (sock < 0) {
 		printf("socket: %s\n", strerror(errno));
@@ -90,7 +70,7 @@ ipv4_recv_sk(char *addr, int port)
 	 */
 	/* mreq.imr_multiaddr.s_addr is set above */
 	mreq.imr_interface.s_addr = htonl(INADDR_ANY);
-	dprintf(4, "Joining multicast group\n");
+	dbg_printf(4, "Joining multicast group\n");
 	if (setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
 		       &mreq, sizeof(mreq)) == -1) {
 		printf("Failed to bind multicast receive socket to "
@@ -100,7 +80,7 @@ ipv4_recv_sk(char *addr, int port)
 		return -1;
 	}
 
-	dprintf(4, "%s: success, fd = %d\n", __FUNCTION__, sock);
+	dbg_printf(4, "%s: success, fd = %d\n", __FUNCTION__, sock);
 	return sock;
 }
 
@@ -110,7 +90,7 @@ ipv4_recv_sk(char *addr, int port)
  */
 int
 ipv4_send_sk(char *send_addr, char *addr, int port, struct sockaddr *tgt,
-	     socklen_t tgt_len)
+	     socklen_t tgt_len, int ttl)
 {
 	int val;
 	struct ip_mreq mreq;
@@ -147,7 +127,7 @@ ipv4_send_sk(char *send_addr, char *addr, int port, struct sockaddr *tgt,
 	/*************************
 	 * SET UP MULTICAST SEND *
 	 *************************/
-	dprintf(4, "Setting up ipv4 multicast send (%s:%d)\n", addr, port);
+	dbg_printf(4, "Setting up ipv4 multicast send (%s:%d)\n", addr, port);
 	sock = socket(PF_INET, SOCK_DGRAM, 0);
 	if (sock < 0) {
 		perror("socket");
@@ -157,7 +137,7 @@ ipv4_send_sk(char *send_addr, char *addr, int port, struct sockaddr *tgt,
 	/*
 	 * Join Multicast group.
 	 */
-	dprintf(4, "Joining IP Multicast group (pass 1)\n");
+	dbg_printf(4, "Joining IP Multicast group (pass 1)\n");
 	if (setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, &mreq,
 		       sizeof(mreq)) == -1) {
 		printf("Failed to add multicast membership to transmit "
@@ -169,7 +149,7 @@ ipv4_send_sk(char *send_addr, char *addr, int port, struct sockaddr *tgt,
 	/*
 	 * Join Multicast group.
 	 */
-	dprintf(4, "Joining IP Multicast group (pass 2)\n");
+	dbg_printf(4, "Joining IP Multicast group (pass 2)\n");
 	if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_IF, &src.sin_addr,
 		       sizeof(src.sin_addr)) == -1) {
 		printf("Failed to bind multicast transmit socket to "
@@ -181,15 +161,15 @@ ipv4_send_sk(char *send_addr, char *addr, int port, struct sockaddr *tgt,
 	/*
 	 * set time to live to 2 hops.
 	 */
-	dprintf(4, "Setting TTL to 2 for fd%d\n", sock);
-	val = 2;
+	dbg_printf(4, "Setting TTL to %d for fd%d\n", ttl, sock);
+	val = ttl;
 	if (setsockopt(sock, SOL_IP, IP_MULTICAST_TTL, &val,
 		       sizeof(val)))
 		printf("warning: setting TTL failed %s\n", strerror(errno));
 
 	memcpy((struct sockaddr_in *)tgt, &mcast, sizeof(struct sockaddr_in));
 
-	dprintf(4, "%s: success, fd = %d\n", __FUNCTION__, sock);
+	dbg_printf(4, "%s: success, fd = %d\n", __FUNCTION__, sock);
 	return sock;
 }
 
@@ -222,7 +202,7 @@ ipv6_recv_sk(char *addr, int port)
 	/********************************
 	 * SET UP MULTICAST RECV SOCKET *
 	 ********************************/
-	dprintf(4, "Setting up ipv6 multicast receive (%s:%d)\n", addr, port);
+	dbg_printf(4, "Setting up ipv6 multicast receive (%s:%d)\n", addr, port);
 	sock = socket(PF_INET6, SOCK_DGRAM, IPPROTO_UDP);
 	if (sock < 0) {
 		printf("socket: %s\n", strerror(errno));
@@ -246,7 +226,7 @@ ipv6_recv_sk(char *addr, int port)
 		return -1;
 	}
 
-	dprintf(4, "Disabling IP Multicast loopback\n");
+	dbg_printf(4, "Disabling IP Multicast loopback\n");
 	val = 1;
 	if (setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_LOOP, &val,
 		       sizeof(val)) != 0) {
@@ -258,7 +238,7 @@ ipv6_recv_sk(char *addr, int port)
 	/*
 	 * Join multicast group
 	 */
-	dprintf(4, "Joining IP Multicast group\n");
+	dbg_printf(4, "Joining IP Multicast group\n");
 	if (setsockopt(sock, IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP, &mreq,
 		       sizeof(mreq)) == -1) {
 		printf("Failed to add multicast to socket %s: %s\n",
@@ -267,7 +247,7 @@ ipv6_recv_sk(char *addr, int port)
 		return -1;
 	}
 
-	dprintf(4, "%s: success, fd = %d\n", __FUNCTION__, sock);
+	dbg_printf(4, "%s: success, fd = %d\n", __FUNCTION__, sock);
 	return sock;
 }
 
@@ -277,7 +257,7 @@ ipv6_recv_sk(char *addr, int port)
  */
 int
 ipv6_send_sk(char *send_addr, char *addr, int port, struct sockaddr *tgt,
-	     socklen_t tgt_len)
+	     socklen_t tgt_len, int ttl)
 {
 	int val;
 	struct ipv6_mreq mreq;
@@ -316,14 +296,14 @@ ipv6_send_sk(char *send_addr, char *addr, int port, struct sockaddr *tgt,
 	/*************************
 	 * SET UP MULTICAST SEND *
 	 *************************/
-	dprintf(4, "Setting up ipv6 multicast send (%s:%d)\n", addr, port);
+	dbg_printf(4, "Setting up ipv6 multicast send (%s:%d)\n", addr, port);
 	sock = socket(PF_INET6, SOCK_DGRAM, 0);
 	if (sock < 0) {
 		perror("socket");
 		return -1;
 	}
 
-	dprintf(4, "Disabling IP Multicast loopback\n");
+	dbg_printf(4, "Disabling IP Multicast loopback\n");
 	val = 1;
 	if (setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_LOOP, &val,
 		       sizeof(val)) != 0) {
@@ -335,7 +315,7 @@ ipv6_send_sk(char *send_addr, char *addr, int port, struct sockaddr *tgt,
 	/*
 	 * Join Multicast group.
 	 */
-	dprintf(4, "Joining IP Multicast group\n");
+	dbg_printf(4, "Joining IP Multicast group\n");
 	if (setsockopt(sock, IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP, &mreq,
 		       sizeof(mreq)) == -1) {
 		printf("Failed to add multicast membership to transmit "
@@ -360,13 +340,13 @@ ipv6_send_sk(char *send_addr, char *addr, int port, struct sockaddr *tgt,
 	/*
 	 * set time to live to 2 hops.
 	 */
-	val = 2;
+	val = ttl;
 	if (setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, &val,
 		       sizeof(val)))
 		printf("warning: setting TTL failed %s\n", strerror(errno));
 
 	memcpy((struct sockaddr_in *)tgt, &mcast, sizeof(struct sockaddr_in6));
 
-	dprintf(4, "%s: success, fd = %d\n", __FUNCTION__, sock);
+	dbg_printf(4, "%s: success, fd = %d\n", __FUNCTION__, sock);
 	return sock;
 }
