@@ -1,17 +1,6 @@
-/*****************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) 2005-2007 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <inttypes.h>
 #include <linux_endian.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -24,6 +13,84 @@
 #include "metawalk.h"
 #include "hash.h"
 
+struct gfs2_inode *get_system_inode(struct gfs2_sbd *sbp, uint64_t block)
+{
+	int j;
+
+	if (block == sbp->md.inum->i_di.di_num.no_addr)
+		return sbp->md.inum;
+	if (block == sbp->md.statfs->i_di.di_num.no_addr)
+		return sbp->md.statfs;
+	if (block == sbp->md.jiinode->i_di.di_num.no_addr)
+		return sbp->md.jiinode;
+	if (block == sbp->md.riinode->i_di.di_num.no_addr)
+		return sbp->md.riinode;
+	if (block == sbp->md.qinode->i_di.di_num.no_addr)
+		return sbp->md.qinode;
+	if (block == sbp->md.pinode->i_di.di_num.no_addr)
+		return sbp->md.pinode;
+	if (block == sbp->md.rooti->i_di.di_num.no_addr)
+		return sbp->md.rooti;
+	if (block == sbp->master_dir->i_di.di_num.no_addr)
+		return sbp->master_dir;
+	if (lf_dip && block == lf_dip->i_di.di_num.no_addr)
+		return lf_dip;
+	for (j = 0; j < sbp->md.journals; j++)
+		if (block == sbp->md.journal[j]->i_di.di_num.no_addr)
+			return sbp->md.journal[j];
+	return NULL;
+}
+
+/* fsck_load_inode - same as gfs2_load_inode() in libgfs2 but system inodes
+   get special treatment. */
+struct gfs2_inode *fsck_load_inode(struct gfs2_sbd *sbp, uint64_t block)
+{
+	struct gfs2_inode *ip = NULL;
+
+	ip = get_system_inode(sbp, block);
+	if (ip) {
+		bhold(ip->i_bh);
+		return ip;
+	}
+	return gfs2_load_inode(sbp, block);
+}
+
+/* fsck_inode_get - same as inode_get() in libgfs2 but system inodes
+   get special treatment. */
+struct gfs2_inode *fsck_inode_get(struct gfs2_sbd *sdp,
+				  struct gfs2_buffer_head *bh)
+{
+	struct gfs2_inode *ip, *sysip;
+
+	zalloc(ip, sizeof(struct gfs2_inode));
+	gfs2_dinode_in(&ip->i_di, bh->b_data);
+	ip->i_bh = bh;
+	ip->i_sbd = sdp;
+
+	sysip = get_system_inode(sdp, ip->i_di.di_num.no_addr);
+	if (sysip) {
+		free(ip);
+		return sysip;
+	}
+	return ip;
+}
+
+/* fsck_inode_put - same as inode_put() in libgfs2 but system inodes
+   get special treatment. */
+void fsck_inode_put(struct gfs2_inode *ip, enum update_flags update)
+{
+	struct gfs2_inode *sysip;
+
+	sysip = get_system_inode(ip->i_sbd, ip->i_di.di_num.no_addr);
+	if (sysip) {
+		if (update)
+			gfs2_dinode_out(&ip->i_di, ip->i_bh->b_data);
+		brelse(ip->i_bh, update);
+	} else {
+		inode_put(ip, update);
+	}
+}
+
 int dirent_repair(struct gfs2_inode *ip, struct gfs2_buffer_head *bh,
 		  struct gfs2_dirent *de, struct gfs2_dirent *dent,
 		  int type, int first)
@@ -140,7 +207,8 @@ int check_entries(struct gfs2_inode *ip, struct gfs2_buffer_head *bh,
 				de.de_inum.no_addr = de.de_inum.no_formal_ino;
 				de.de_inum.no_formal_ino = 0;
 				gfs2_dirent_out(&de, (char *)dent);
-				*update = 1; /* Mark dirent buffer as modified */
+				*update = (opts.no ? not_updated : updated);
+				/* Mark dirent buffer as modified */
 				first = 0;
 			}
 			else {
@@ -282,7 +350,7 @@ int check_leaf(struct gfs2_inode *ip, int *update, struct metawalk_fxns *pass)
 					       old_leaf, index,
 					       "that is not really a leaf");
 				memcpy(&leaf, &oldleaf, sizeof(oldleaf));
-				brelse(lbh, updated);
+				brelse(lbh, (opts.no ? not_updated : updated));
 				break;
 			}
 			gfs2_leaf_in(&leaf, lbh->b_data);
@@ -303,7 +371,7 @@ int check_leaf(struct gfs2_inode *ip, int *update, struct metawalk_fxns *pass)
 				leaf.lf_dirent_format = GFS2_FORMAT_DE;
 				gfs2_leaf_out(&leaf, lbh->b_data);
 				log_debug("Fixing lf_dirent_format.\n");
-				*update = updated;
+				*update = (opts.no ? not_updated : updated);
 			}
 
 			/* Make sure it's really a leaf. */
@@ -462,9 +530,11 @@ static int check_leaf_eattr(struct gfs2_inode *ip, uint64_t block,
 		if(error > 0) {
 			return 1;
 		}
+		check_eattr_entries(ip, bh, pass);
+		if (bh)
+			brelse(bh, not_updated);
 	}
 
-	check_eattr_entries(ip, bh, pass);
 	return 0;
 }
 
@@ -486,9 +556,9 @@ static int check_indirect_eattr(struct gfs2_inode *ip, uint64_t indirect,
 	log_debug("Checking EA indirect block #%"PRIu64" (0x%" PRIx64 ").\n",
 			  indirect, indirect);
 
-	if (!pass->check_eattr_indir ||
+	if (pass->check_eattr_indir &&
 	    !pass->check_eattr_indir(ip, indirect, ip->i_di.di_num.no_addr,
-								 &indirect_buf, pass->private)) {
+				     &indirect_buf, pass->private)) {
 		ea_leaf_ptr = (uint64_t *)(indirect_buf->b_data
 								   + sizeof(struct gfs2_meta_header));
 		end = ea_leaf_ptr + ((sdp->sd_sb.sb_bsize
@@ -539,83 +609,84 @@ int check_inode_eattr(struct gfs2_inode *ip, struct metawalk_fxns *pass)
 
 /**
  * build_and_check_metalist - check a bunch of indirect blocks
+ * Note: Every buffer put on the metalist should be "held".
  * @ip:
  * @mlp:
  */
 static int build_and_check_metalist(struct gfs2_inode *ip,
-									struct metawalk_fxns *pass)
+				    osi_list_t *mlp,
+				    struct metawalk_fxns *pass)
 {
 	uint32_t height = ip->i_di.di_height;
-	struct gfs2_buffer_head *nbh, *metabh;
-	int head_size;
+	struct gfs2_buffer_head *bh, *nbh, *metabh;
+	osi_list_t *prev_list, *cur_list, *tmp;
+	int i, head_size;
 	uint64_t *ptr, block;
 	int err;
 
 	metabh = bread(ip->i_sbd, ip->i_di.di_num.no_addr);
 
-	/* if(<there are no indirect blocks to check>) */
-	if (height < 1) {
-		brelse(metabh, not_updated);
-		return 0;
-	}
-	head_size = sizeof(struct gfs2_dinode);
-	for (ptr = (uint64_t *)(metabh->b_data + head_size);
-		 (char *)ptr < (metabh->b_data + metabh->b_size); ptr++) {
-		nbh = NULL;
-		
-		if (!*ptr)
-			continue;
+	osi_list_add(&metabh->b_altlist, &mlp[0]);
 
-		block = be64_to_cpu(*ptr);
-		if (height > 1) {
-			uint64_t *subptr, subblock, prevsubblock;
-			int head_size2;
-			struct gfs2_buffer_head *nbh2;
-			
-			nbh2 = NULL;
-			prevsubblock = 0;
-			head_size2 = sizeof(struct gfs2_meta_header);
-			err = pass->check_metalist(ip, block, &nbh, pass->private);
-			if (!nbh)
-				nbh = nbh2 = bread(ip->i_sbd, block);
-			for (subptr = (uint64_t *)(nbh->b_data + head_size2);
-				 (char *)subptr < (nbh->b_data + nbh->b_size);
-				 subptr++) {
+	/* if(<there are no indirect blocks to check>) */
+	if (height < 2)
+		return 0;
+	for (i = 1; i < height; i++){
+		prev_list = &mlp[i - 1];
+		cur_list = &mlp[i];
+
+		for (tmp = prev_list->next; tmp != prev_list; tmp = tmp->next){
+			bh = osi_list_entry(tmp, struct gfs2_buffer_head,
+					    b_altlist);
+
+			head_size = (i > 1 ?
+				     sizeof(struct gfs2_meta_header) :
+				     sizeof(struct gfs2_dinode));
+
+			for (ptr = (uint64_t *)(bh->b_data + head_size);
+			     (char *)ptr < (bh->b_data + bh->b_size);
+			     ptr++) {
+				nbh = NULL;
+		
+				if (!*ptr)
+					continue;
+
+				block = be64_to_cpu(*ptr);
+				err = pass->check_metalist(ip, block, &nbh,
+							   pass->private);
+				/* check_metalist should hold any buffers
+				   it gets with "bread". */
 				if(err < 0) {
 					stack;
 					goto fail;
 				}
 				if(err > 0) {
-					log_debug("Skipping block %" PRIu64 " (0x%" PRIx64 ")\n",
-							  block, block);
+					log_debug("Skipping block %" PRIu64
+						  " (0x%" PRIx64 ")\n",
+						  block, block);
 					continue;
 				}
-				subblock = be64_to_cpu(*subptr);
-				if (subblock) {
-					if (subblock != prevsubblock) {
-						prevsubblock = subblock;
-						if(pass->check_data &&
-						   (pass->check_data(ip, subblock,
-											 pass->private) < 0)) {
-							stack;
-							return -1;
-						}
-					} /* if different than the previous block */
-				} /* if non-zero indirect block pointer */
+				if(!nbh)
+					nbh = bread(ip->i_sbd, block);
+
+				osi_list_add(&nbh->b_altlist, cur_list);
 			} /* for all data on the indirect block */
-			if (nbh2)
-				brelse(nbh2, not_updated);
-		} /* if height */
-		else if(pass->check_data &&
-				(pass->check_data(ip, block, pass->private) < 0)) {
-			stack;
-			brelse(metabh, not_updated);
-			return -1;
-		}
-	} /* for all level 1 pointers */
-fail:
-	brelse(metabh, not_updated);
+		} /* for blocks at that height */
+	} /* for height */
 	return 0;
+fail:
+	for (i = 0; i < GFS2_MAX_META_HEIGHT; i++) {
+		osi_list_t *list;
+		list = &mlp[i];
+		while (!osi_list_empty(list)) {
+			nbh = osi_list_entry(list->next,
+					     struct gfs2_buffer_head, b_altlist);
+			osi_list_del(&nbh->b_altlist);
+		}
+	}
+	/* This is an error path, so we need to release the buffer here: */
+	brelse(metabh, not_updated);
+	return -1;
 }
 
 /**
@@ -626,18 +697,72 @@ fail:
  */
 int check_metatree(struct gfs2_inode *ip, struct metawalk_fxns *pass)
 {
+	osi_list_t metalist[GFS2_MAX_META_HEIGHT];
+	osi_list_t *list, *tmp;
+	struct gfs2_buffer_head *bh;
+	uint64_t block, *ptr;
 	uint32_t height = ip->i_di.di_height;
+	int  i, head_size;
 	int update = 0;
 	int error = 0;
 
-	if (height) {
-		/* create metalist for each level */
-		if(build_and_check_metalist(ip, pass)){
-			stack;
-			return -1;
+	if (!height)
+		goto end;
+
+	for (i = 0; i < GFS2_MAX_META_HEIGHT; i++)
+		osi_list_init(&metalist[i]);
+
+	/* create metalist for each level */
+	if (build_and_check_metalist(ip, &metalist[0], pass)){
+		stack;
+		return -1;
+	}
+
+	/* We don't need to record directory blocks - they will be
+	 * recorded later...i think... */
+        if (S_ISDIR(ip->i_di.di_mode))
+		log_debug("Directory with height > 0 at %"PRIu64"\n",
+			  ip->i_di.di_num.no_addr);
+
+	/* check data blocks */
+	list = &metalist[height - 1];
+
+	for (tmp = list->next; tmp != list; tmp = tmp->next) {
+		bh = osi_list_entry(tmp, struct gfs2_buffer_head, b_altlist);
+
+		head_size = (height != 1 ? sizeof(struct gfs2_meta_header) :
+			     sizeof(struct gfs2_dinode));
+		ptr = (uint64_t *)(bh->b_data + head_size);
+
+		for ( ; (char *)ptr < (bh->b_data + bh->b_size); ptr++)	{
+			if (!*ptr)
+				continue;
+
+			block =  be64_to_cpu(*ptr);
+
+			if(pass->check_data &&
+			   (pass->check_data(ip, block, pass->private) < 0)) {
+				stack;
+				return -1;
+			}
 		}
 	}
-	if (S_ISDIR(ip->i_di.di_mode)) {
+
+	/* free metalists */
+	for (i = 0; i < GFS2_MAX_META_HEIGHT; i++)
+	{
+		list = &metalist[i];
+		while (!osi_list_empty(list))
+		{
+			bh = osi_list_entry(list->next,
+					    struct gfs2_buffer_head, b_altlist);
+			brelse(bh, not_updated);
+			osi_list_del(&bh->b_altlist);
+		}
+	}
+
+end:
+        if (S_ISDIR(ip->i_di.di_mode)) {
 		/* check validity of leaf blocks and leaf chains */
 		if (ip->i_di.di_flags & GFS2_DIF_EXHASH) {
 			error = check_leaf(ip, &update, pass);
@@ -676,13 +801,13 @@ int check_dir(struct gfs2_sbd *sbp, uint64_t block, struct metawalk_fxns *pass)
 	int error = 0;
 
 	bh = bread(sbp, block);
-	ip = inode_get(sbp, bh);
+	ip = fsck_inode_get(sbp, bh);
 
 	if(ip->i_di.di_flags & GFS2_DIF_EXHASH) {
 		error = check_leaf(ip, &update, pass);
 		if(error < 0) {
 			stack;
-			inode_put(ip, not_updated); /* does brelse(bh); */
+			fsck_inode_put(ip, not_updated); /* does brelse(bh); */
 			return -1;
 		}
 	}
@@ -690,12 +815,12 @@ int check_dir(struct gfs2_sbd *sbp, uint64_t block, struct metawalk_fxns *pass)
 		error = check_linear_dir(ip, bh, &update, pass);
 		if(error < 0) {
 			stack;
-			inode_put(ip, not_updated); /* does brelse(bh); */
+			fsck_inode_put(ip, not_updated); /* does brelse(bh); */
 			return -1;
 		}
 	}
 
-	inode_put(ip, update); /* does a brelse */
+	fsck_inode_put(ip, opts.no ? not_updated : update); /* does a brelse */
 	return error;
 }
 
@@ -713,15 +838,12 @@ static int remove_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 	memset(&dentry, 0, sizeof(struct gfs2_dirent));
 	gfs2_dirent_in(&dentry, (char *)dent);
 	de = &dentry;
+	*update = (opts.no ? not_updated : updated);
 
-	if(de->de_inum.no_addr == *dentryblock) {
-		*update = 1;
+	if(de->de_inum.no_addr == *dentryblock)
 		dirent2_del(ip, bh, prev_de, dent);
-	}
-	else {
+	else
 		(*count)++;
-		*update = 1;
-	}
 
 	return 0;
 
