@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <dirent.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -89,18 +76,13 @@ int set_dotdot_dir(struct gfs2_sbd *sbp, uint64_t childblock,
 static int check_eattr_indir(struct gfs2_inode *ip, uint64_t block,
 			    uint64_t parent, struct gfs2_buffer_head **bh, void *private)
 {
-
+	*bh = bread(ip->i_sbd, block);
 	return 0;
 }
 static int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
 			    uint64_t parent, struct gfs2_buffer_head **bh, void *private)
 {
-#if 0
-	struct gfs2_buffer_head *leaf_bh;
-
-	leaf_bh = bread(ip->i_sbd, block);
-	brelse(leaf_bh);
-#endif
+	*bh = bread(ip->i_sbd, block);
 	return 0;
 }
 
@@ -157,9 +139,9 @@ static int check_file_type(uint8_t de_type, uint8_t block_type)
 /* FIXME: should maybe refactor this a bit - but need to deal with
  * FIXMEs internally first */
 int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
-				 struct gfs2_dirent *prev_de,
-				 struct gfs2_buffer_head *bh, char *filename, int *update,
-				 uint16_t *count, void *priv)
+		 struct gfs2_dirent *prev_de,
+		 struct gfs2_buffer_head *bh, char *filename,
+		 int *update, uint16_t *count, void *priv)
 {
 	struct gfs2_sbd *sbp = ip->i_sbd;
 	struct gfs2_block_query q = {0};
@@ -231,10 +213,10 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 				 "Clear directory entry tp out of range block? (y/n) ")) {
 			log_err("Clearing %s\n", tmp_name);
 			dirent2_del(ip, bh, prev_de, dent);
+			*update = 1;
 			return 1;
 		} else {
 			log_err("Directory entry to out of range block remains\n");
-			*update = 1;
 			(*count)++;
 			ds->entry_count++;
 			return 0;
@@ -255,9 +237,9 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 
 		if(query(&opts, "Clear entry to inode containing bad blocks? (y/n)")) {
 
-			entry_ip = gfs2_load_inode(sbp, de->de_inum.no_addr);
+			entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
 			check_inode_eattr(entry_ip, &clear_eattrs);
-			inode_put(entry_ip, not_updated);
+			fsck_inode_put(entry_ip, not_updated);
 
 			/* FIXME: make sure all blocks referenced by
 			 * this inode are cleared in the bitmap */
@@ -265,10 +247,10 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 			dirent2_del(ip, bh, prev_de, dent);
 
 			gfs2_block_set(bl, de->de_inum.no_addr, gfs2_meta_inval);
+			*update = 1;
 			return 1;
 		} else {
 			log_warn("Entry to inode containing bad blocks remains\n");
-			*update = 1;
 			(*count)++;
 			ds->entry_count++;
 			return 0;
@@ -291,11 +273,11 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 			 * this inode are cleared in the bitmap */
 
 			dirent2_del(ip, bh, prev_de, dent);
+			*update = 1;
 			log_warn("Directory entry '%s' cleared\n", tmp_name);
 			return 1;
 		} else {
 			log_err("Directory entry to non-inode block remains\n");
-			*update = 1;
 			(*count)++;
 			ds->entry_count++;
 			return 0;
@@ -314,15 +296,15 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 				 de->de_inum.no_addr, de->de_inum.no_addr,
 				 block_type_string(&q));
 		if(query(&opts, "Clear stale directory entry? (y/n) ")) {
-			entry_ip = gfs2_load_inode(sbp, de->de_inum.no_addr);
+			entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
 			check_inode_eattr(entry_ip, &clear_eattrs);
-			inode_put(entry_ip, not_updated);
+			fsck_inode_put(entry_ip, not_updated);
 
 			dirent2_del(ip, bh, prev_de, dent);
+			*update  = 1;
 			return 1;
 		} else {
 			log_err("Stale directory entry remains\n");
-			*update  = 1;
 			(*count)++;
 			ds->entry_count++;
 			return 0;
@@ -338,11 +320,12 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 					ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
 			if(query(&opts, "Clear duplicate '.' entry? (y/n) ")) {
 
-				entry_ip = gfs2_load_inode(sbp, de->de_inum.no_addr);
+				entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
 				check_inode_eattr(entry_ip, &clear_eattrs);
-				inode_put(entry_ip, not_updated);
+				fsck_inode_put(entry_ip, not_updated);
 
 				dirent2_del(ip, bh, prev_de, dent);
+				*update  = 1;
 				return 1;
 			} else {
 				log_err("Duplicate '.' entry remains\n");
@@ -350,7 +333,6 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 				 * and check the rest of the '.'
 				 * entry? */
 				increment_link(sbp, de->de_inum.no_addr);
-				*update  = 1;
 				(*count)++;
 				ds->entry_count++;
 				return 0;
@@ -369,11 +351,12 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 					de->de_inum.no_addr, de->de_inum.no_addr,
 					ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
 			if(query(&opts, "Remove '.' reference? (y/n) ")) {
-				entry_ip = gfs2_load_inode(sbp, de->de_inum.no_addr);
+				entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
 				check_inode_eattr(entry_ip, &clear_eattrs);
-				inode_put(entry_ip, not_updated);
+				fsck_inode_put(entry_ip, not_updated);
 
 				dirent2_del(ip, bh, prev_de, dent);
+				*update = 1;
 				return 1;
 
 			} else {
@@ -381,7 +364,6 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 				/* Not setting ds->dotdir here since
 				 * this '.' entry is invalid */
 				increment_link(sbp, de->de_inum.no_addr);
-				*update = 1;
 				(*count)++;
 				ds->entry_count++;
 				return 0;
@@ -390,7 +372,7 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 
 		ds->dotdir = 1;
 		increment_link(sbp, de->de_inum.no_addr);
-		*update = 1;
+		*update = (opts.no ? not_updated : updated);
 		(*count)++;
 		ds->entry_count++;
 
@@ -404,9 +386,9 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 					ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
 			if(query(&opts, "Clear duplicate '..' entry? (y/n) ")) {
 
-				entry_ip = gfs2_load_inode(sbp, de->de_inum.no_addr);
+				entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
 				check_inode_eattr(entry_ip, &clear_eattrs);
-				inode_put(entry_ip, not_updated);
+				fsck_inode_put(entry_ip, not_updated);
 
 				dirent2_del(ip, bh, prev_de, dent);
 				*update = 1;
@@ -417,7 +399,6 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 				 * and check the rest of the '..'
 				 * entry? */
 				increment_link(sbp, de->de_inum.no_addr);
-				*update  = 1;
 				(*count)++;
 				ds->entry_count++;
 				return 0;
@@ -430,9 +411,9 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 					" something that's not a directory",
 					ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
 			if(query(&opts, "Clear bad '..' directory entry? (y/n) ")) {
-				entry_ip = gfs2_load_inode(sbp, de->de_inum.no_addr);
+				entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
 				check_inode_eattr(entry_ip, &clear_eattrs);
-				inode_put(entry_ip, not_updated);
+				fsck_inode_put(entry_ip, not_updated);
 
 				dirent2_del(ip, bh, prev_de, dent);
 				*update = 1;
@@ -440,7 +421,6 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 			} else {
 				log_err("Bad '..' directory entry remains\n");
 				increment_link(sbp, de->de_inum.no_addr);
-				*update  = 1;
 				(*count)++;
 				ds->entry_count++;
 				return 0;
@@ -459,7 +439,7 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 
 		ds->dotdotdir = 1;
 		increment_link(sbp, de->de_inum.no_addr);
-		*update = 1;
+		*update = (opts.no ? not_updated : updated);
 		(*count)++;
 		ds->entry_count++;
 		return 0;
@@ -470,7 +450,7 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 	if(q.block_type != gfs2_inode_dir) {
 		log_debug("Found non-dir inode dentry\n");
 		increment_link(sbp, de->de_inum.no_addr);
-		*update = 1;
+		*update = (opts.no ? not_updated : updated);
 		(*count)++;
 		ds->entry_count++;
 		return 0;
@@ -491,7 +471,6 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 			return 1;
 		} else {
 			log_err("Hard link to directory remains\n");
-			*update = 1;
 			(*count)++;
 			ds->entry_count++;
 			return 0;
@@ -502,7 +481,7 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 		return -1;
 	}
 	increment_link(sbp, de->de_inum.no_addr);
-	*update = 1;
+	*update = (opts.no ? not_updated : updated);
 	(*count)++;
 	ds->entry_count++;
 	/* End of checks */
@@ -608,7 +587,7 @@ int check_system_dir(struct gfs2_inode *sysinode, const char *dirname,
 		}
 	}
 
-	brelse(bh, update);
+	brelse(bh, opts.no ? not_updated : update);
 	return 0;
 }
 
@@ -693,13 +672,13 @@ int pass2(struct gfs2_sbd *sbp)
 		if(ds.q.bad_block) {
 			/* First check that the directory's metatree
 			 * is valid */
-			ip = gfs2_load_inode(sbp, i);
+			ip = fsck_load_inode(sbp, i);
 			if(check_metatree(ip, &pass2_fxns)) {
 				stack;
 				free(ip);
 				return -1;
 			}
-			inode_put(ip, not_updated);
+			fsck_inode_put(ip, not_updated);
 		}
 		error = check_dir(sbp, i, &pass2_fxns);
 		if(error < 0) {
@@ -738,7 +717,7 @@ int pass2(struct gfs2_sbd *sbp)
 			gfs2_block_set(bl, i, gfs2_meta_inval);
 		}
 		bh = bread(sbp, i);
-		ip = inode_get(sbp, bh);
+		ip = fsck_inode_get(sbp, bh);
 		if(!ds.dotdir) {
 			log_err("No '.' entry found\n");
 			sprintf(tmp_name, ".");
@@ -761,21 +740,20 @@ int pass2(struct gfs2_sbd *sbp)
 			free(filename);
 
 		}
-		inode_put(ip, not_updated); /* does a brelse */
+		fsck_inode_put(ip, not_updated); /* does a brelse */
 
 		bh = bread(sbp, i);
-		ip = inode_get(sbp, bh);
+		ip = fsck_inode_get(sbp, bh);
 		if(ip->i_di.di_entries != ds.entry_count) {
 			log_err("Entries is %d - should be %d for inode block %" PRIu64
 					" (0x%" PRIx64 ")\n",
 					ip->i_di.di_entries, ds.entry_count,
 					ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
 			ip->i_di.di_entries = ds.entry_count;
-			gfs2_dinode_out(&ip->i_di, bh->b_data);
-			inode_put(ip, updated); /* does a brelse */
+			fsck_inode_put(ip, updated); /* does a gfs2_dinode_out, brelse */
 		}
 		else
-			inode_put(ip, not_updated); /* does a brelse */
+			fsck_inode_put(ip, not_updated); /* does a brelse */
 	}
 	return 0;
 }
