@@ -1,15 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -77,6 +65,7 @@ print_usage()
 	printf("  warn             set a quota warning value for an ID\n");
 	printf("  check            check the quota file\n");
 	printf("  init             initialize the quota file\n");
+	printf("  reset            reset the quota file\n");
 	printf("\n");
 	printf("Options:\n");
 	printf("  -b               sizes are in FS blocks\n");
@@ -157,7 +146,7 @@ decode_arguments(int argc, char *argv[], commandline_t *comline)
 			break;
 
 		case 'V':
-			printf("gfs2_quota %s (built %s %s)\n", GFS2_RELEASE_NAME,
+			printf("gfs2_quota %s (built %s %s)\n", RELEASE_VERSION,
 			       __DATE__, __TIME__);
 			printf("%s\n", REDHAT_COPYRIGHT);
 			exit(EXIT_SUCCESS);
@@ -214,6 +203,10 @@ decode_arguments(int argc, char *argv[], commandline_t *comline)
 			if (comline->operation)
 				die("can't specify two operations\n");
 			comline->operation = GQ_OP_INIT;
+		} else if (strcmp(argv[optind], "reset") == 0) {
+			if (comline->operation)
+				die("can't specify two operations\n");
+			comline->operation = GQ_OP_RESET;
 		} else
 			die("unknown option %s\n", argv[optind]);
 
@@ -250,26 +243,34 @@ print_quota(commandline_t *comline,
 
 	case GQ_UNITS_KILOBYTE:
 		if (sb->sb_bsize == 512)
-			printf("limit: %-10"PRIu64" warn: %-10"PRIu64"value: %-10"PRId64"\n",
-			       q->qu_limit / 2,
-			       q->qu_warn / 2,
-			       q->qu_value / 2);
+			printf("limit: %-10llu warn: %-10lluvalue: %-10llu\n",
+			       (unsigned long long)q->qu_limit / 2,
+			       (unsigned long long)q->qu_warn / 2,
+			       (unsigned long long)q->qu_value / 2);
 		else
-			printf("limit: %-10"PRIu64" warn: %-10"PRIu64"value: %-10"PRId64"\n",
+			printf("limit: %-10llu warn: %-10lluvalue: %-10llu\n",
+			       (unsigned long long)
 			       q->qu_limit << (sb->sb_bsize_shift - 10),
+			       (unsigned long long)
 			       q->qu_warn << (sb->sb_bsize_shift - 10),
+			       (unsigned long long)
 			       q->qu_value << (sb->sb_bsize_shift - 10));
 		break;
 
 	case GQ_UNITS_FSBLOCK:
-		printf("limit: %-10"PRIu64" warn: %-10"PRIu64" value: %-10"PRId64"\n",
-		       q->qu_limit, q->qu_warn, q->qu_value);
+		printf("limit: %-10llu warn: %-10llu value: %-10llu\n",
+		       (unsigned long long)q->qu_limit,
+		       (unsigned long long)q->qu_warn,
+		       (unsigned long long)q->qu_value);
 		break;
 
 	case GQ_UNITS_BASICBLOCK:
-		printf("limit: %-10"PRIu64" warn: %-10"PRIu64" value: %-10"PRId64"\n",
+		printf("limit: %-10llu warn: %-10llu value: %-10llu\n",
+		       (unsigned long long)
 		       q->qu_limit << (sb->sb_bsize_shift - 9),
+		       (unsigned long long)
 		       q->qu_warn << (sb->sb_bsize_shift - 9),
+		       (unsigned long long)
 		       q->qu_value << (sb->sb_bsize_shift - 9));
 		break;
 
@@ -297,6 +298,240 @@ read_superblock(struct gfs2_sb *sb, struct gfs2_sbd *sdp)
 	close(fd);
 }
 
+inline void 
+read_quota_internal(int fd, uint32_t id, int id_type, struct gfs2_quota *q)
+{
+	/* seek to the appropriate offset in the quota file and read the 
+	   quota info */
+	uint64_t offset;
+	char buf[256];
+	int error;
+	if (id_type == GQ_ID_USER)
+		offset = (2 * (uint64_t)id) * sizeof(struct gfs2_quota);
+	else
+		offset = (2 * (uint64_t)id + 1) * sizeof(struct gfs2_quota);
+	lseek(fd, offset, SEEK_SET);
+	error = read(fd, buf, sizeof(struct gfs2_quota));
+	if (error < 0)
+		die("failed to read from quota file: %s\n", strerror(errno));
+	if (error != sizeof(struct gfs2_quota))
+		die("Couldn't read %lu bytes from quota file at offset %llu\n",
+		    (unsigned long)sizeof(struct gfs2_quota),
+		    (unsigned long long)offset);
+	gfs2_quota_in(q, buf);
+}
+
+inline void 
+write_quota_internal(int fd, uint32_t id, int id_type, struct gfs2_quota *q)
+{
+	/* seek to the appropriate offset in the quota file and read the
+	   quota info */
+	uint64_t offset;
+	char buf[256];
+	int error;
+	if (id_type == GQ_ID_USER)
+		offset = (2 * (uint64_t)id) * sizeof(struct gfs2_quota);
+	else
+		offset = (2 * (uint64_t)id + 1) * sizeof(struct gfs2_quota);
+	lseek(fd, offset, SEEK_SET);
+	gfs2_quota_out(q, buf);
+	error = write(fd, buf, sizeof(struct gfs2_quota));
+	if (error != sizeof(struct gfs2_quota))
+		die("failed to write to quota file: %s\n", strerror(errno));
+}
+
+/**
+ * get_last_quota_id - Get the last quota in the quota file
+ * @fd: an open file descriptor of the quota file
+ * @id_type: GQ_ID_USER or GQ_ID_GROUP
+ * @max_id: return the maximum id obtained
+ */
+void 
+get_last_quota_id(int fd, uint32_t *max_id)
+{
+	/* stat(2) the quota file to find how big it is. This will give us a
+	 * a rough idea of what the last valid quota uid/gid is. It is possible 
+	 * that the last quota in the file belongs to a group with gid:x and 
+	 * the corresponding user quota with uid:x doesn't exist. In such cases
+	 * we still return x as max_id. This max_id is ONLY A HINT. If used 
+	 * as a terminating condition of a loop, another condition should also
+	 * be specified.
+	 */
+	struct stat st;
+	uint32_t qsize = sizeof(struct gfs2_quota);
+	uint64_t size;
+	if (fstat(fd, &st))
+		die("failed to stat the quota file: %s\n", strerror(errno));
+	size = st.st_size;
+	if (!size)
+		die("error: quota file is truncated to zero!\n");
+	if (size % qsize) {
+		printf("warning: quota file size not a multiple of "
+		       "struct gfs2_quota\n");
+		size = qsize * (size / qsize);
+	}
+	*max_id = (size - 1) / (2 * qsize);
+}
+
+/**
+ * is_valid_quota_list - Check if we have a valid quota list
+ * @fd: an open file descriptor of the quota file
+ * Returns 0 or 1.
+ */
+int 
+is_valid_quota_list(int fd)
+{
+	/* This is a slow test to determine if the quotas are in a 
+	 * linked list. We should come up with something better
+	 * Quota linked list format is identified by the following.
+	 * step1: Get the maximum groupid and userid having valid
+	 *        quotas in the quota file.
+	 * step2: Obtain the size of the quota file. The size of the 
+	 *        quota file (position of the last valid quota) 
+	 *        determines the last user/group id.
+	 * step3: If we can obtain the last valid quota through the 
+	 *        lists, then our lists are good. Else, the lists are 
+	 *        either corrupt or an older quota file format is in use
+	 */
+	int id_type = GQ_ID_GROUP;
+	uint32_t id = 0, prev, ulast = 0, glast = 0, max;
+	struct gfs2_quota q;
+
+	get_last_quota_id(fd, &max);
+again:
+	do {
+		read_quota_internal(fd, id, id_type, &q);
+		prev = id;
+		id = q.qu_ll_next;
+		if (id > max)
+			return 0;
+	} while (id && id > prev);
+
+	if (id && id <= prev)
+		return 0;
+
+	if (id_type == GQ_ID_GROUP)
+		glast = prev;
+	else
+		ulast = prev;
+
+	if (id_type == GQ_ID_GROUP) {
+		id_type = GQ_ID_USER;
+		id = 0;
+		goto again;
+	}
+
+	if (glast != max && ulast != max)
+		return 0;
+	
+	return 1;
+}
+
+void 
+print_quota_list_warning()
+{
+	printf("\nWarning: This filesystem doesn't seem to have the new quota "
+	       "list format or the quota list is corrupt. list, check and init "
+	       "operation performance will suffer due to this. It is recommended "
+	       "that you run the 'gfs2_quota reset' operation to reset the quota "
+	       "file. All current quota information will be lost and you will "
+	       "have to reassign all quota limits and warnings\n\n"); 
+}
+
+/**
+ * adjust_quota_list - Adjust the quota linked list
+ * @fd: The quota file descriptor
+ * @comline: the struct containing the parsed command line arguments
+ */
+static void
+adjust_quota_list(int fd, commandline_t *comline)
+{
+	uint32_t prev = 0, next = 0, id = comline->id;
+	struct gfs2_quota tmpq, q;
+	int id_type = comline->id_type;
+	
+	if (id == 0) /* root quota, don't do anything */
+		goto out;
+	/* We just wrote the quota for id in do_set(). Get it */
+	next = 0;
+	do {
+		read_quota_internal(fd, next, id_type, &q);
+		prev = next;
+		next = q.qu_ll_next;
+		if (prev == id) /* no duplicates, bail */
+			goto out;
+		if (prev < id && id < next) /* gotcha! */
+			break;
+	} while(next && next > prev);
+	read_quota_internal(fd, id, id_type, &tmpq);
+	tmpq.qu_ll_next = next;
+	q.qu_ll_next = id;
+	write_quota_internal(fd, id, id_type, &tmpq);
+	write_quota_internal(fd, prev, id_type, &q);
+
+out:
+	return;
+}
+
+/**
+ * do_reset - Reset all the quota data for a filesystem
+ * @comline: the struct containing the parsed command line arguments
+ */
+
+static void
+do_reset(struct gfs2_sbd *sdp, commandline_t *comline)
+{
+	int fd;
+	char quota_file[BUF_SIZE], c;
+	struct gfs2_quota q;
+
+	if (!*comline->filesystem)
+		die("need a filesystem to work on\n");
+
+	printf("This operation will permanently erase all quota information. "
+	       "You will have to re-assign all quota limit/warn values. "
+	       "Proceed [y/N]? ");
+	c = getchar();
+	if (c != 'y' && c != 'Y')
+		return;
+
+	strcpy(sdp->path_name, comline->filesystem);
+	check_for_gfs2(sdp);
+	read_superblock(&sdp->sd_sb, sdp);
+	if (!find_gfs2_meta(sdp))
+		mount_gfs2_meta(sdp);
+	lock_for_admin(sdp);
+	
+	strcpy(quota_file, sdp->metafs_path);
+	strcat(quota_file, "/quota");
+
+	fd = open(quota_file, O_RDWR);
+	if (fd < 0) {
+		close(sdp->metafs_fd);
+		cleanup_metafs(sdp);
+		die("can't open file %s: %s\n", quota_file,
+		    strerror(errno));
+	}
+
+	read_quota_internal(fd, 0, GQ_ID_USER, &q);
+	q.qu_ll_next = 0;
+	write_quota_internal(fd, 0, GQ_ID_USER, &q);
+
+	read_quota_internal(fd, 0, GQ_ID_GROUP, &q);
+	q.qu_ll_next = 0;
+	write_quota_internal(fd, 0, GQ_ID_GROUP, &q);
+
+	/* truncate the quota file such that only the first
+	 * two quotas(uid=0 and gid=0) remain.
+	 */
+	if (ftruncate(fd, (sizeof(struct gfs2_quota)) * 2))
+		die("couldn't truncate quota file %s\n", strerror(errno));
+	
+	close(fd);
+	close(sdp->metafs_fd);
+	cleanup_metafs(sdp);
+}
+
 /**
  * do_list - List all the quota data for a filesystem
  * @comline: the struct containing the parsed command line arguments
@@ -310,10 +545,11 @@ do_list(struct gfs2_sbd *sdp, commandline_t *comline)
 	struct gfs2_quota q;
 	char buf[sizeof(struct gfs2_quota)];
 	uint64_t offset;
-	uint32_t id;
+	uint32_t id, prev, maxid;
 	int pass = 0;
 	int error = 0;
 	char quota_file[BUF_SIZE];
+	int id_type = comline->id_type;
 	
 	if (!*comline->filesystem)
 		die("need a filesystem to work on\n");
@@ -335,6 +571,30 @@ do_list(struct gfs2_sbd *sdp, commandline_t *comline)
 		die("can't open file %s: %s\n", quota_file,
 		    strerror(errno));
 	}
+	
+	if (!is_valid_quota_list(fd)) {
+		print_quota_list_warning();
+		goto do_old_school;
+	}
+	get_last_quota_id(fd, &maxid);
+	
+	for (pass=0; pass<2; pass++) {
+		id = 0;
+		id_type = pass ? GQ_ID_GROUP : GQ_ID_USER;
+		
+		do {
+			read_quota_internal(fd, id, id_type, &q);
+			prev = id;
+			if (q.qu_limit || q.qu_warn || q.qu_value)
+				print_quota(comline, 
+					    id_type == GQ_ID_USER ? TRUE : FALSE, 
+					    id, &q, &sdp->sd_sb);
+			id = q.qu_ll_next;
+		} while(id && id > prev && id <= maxid);
+	}
+	goto out;
+
+do_old_school:
 	for (pass=0; pass<2; pass++) {
 		if (!pass)
 			offset = 0;
@@ -359,7 +619,7 @@ do_list(struct gfs2_sbd *sdp, commandline_t *comline)
 			offset += 2 * sizeof(struct gfs2_quota);
 		} while (error == sizeof(struct gfs2_quota));
 	}
-
+out:
 	close(fd);
 	close(sdp->metafs_fd);
 	cleanup_metafs(sdp);
@@ -380,6 +640,7 @@ do_get_one(struct gfs2_sbd *sdp, commandline_t *comline, char *filesystem)
 	struct gfs2_quota q;
 	uint64_t offset;
 	int error;
+	uint32_t maxid;
 	char quota_file[BUF_SIZE];
 
 	strcpy(sdp->path_name, filesystem);
@@ -407,6 +668,10 @@ do_get_one(struct gfs2_sbd *sdp, commandline_t *comline, char *filesystem)
 
 	memset(&q, 0, sizeof(struct gfs2_quota));
 	
+	get_last_quota_id(fd, &maxid);
+	if (comline->id > maxid)
+		goto print_empty_quota;
+
 	lseek(fd, offset, SEEK_SET);
 	error = read(fd, buf, sizeof(struct gfs2_quota));
 	if (error < 0) {
@@ -420,6 +685,7 @@ do_get_one(struct gfs2_sbd *sdp, commandline_t *comline, char *filesystem)
 	gfs2_quota_in(&q, buf);
 
 
+print_empty_quota:
 	print_quota(comline,
 		    (comline->id_type == GQ_ID_USER), comline->id,
 		    &q, &sdp->sd_sb);
@@ -477,24 +743,10 @@ do_get(struct gfs2_sbd *sdp, commandline_t *comline)
 static void 
 do_sync_one(struct gfs2_sbd *sdp, char *filesystem)
 {
-	int fd;
-	char sys_quota_sync[PATH_MAX];
+	char *fsname;
 
-	strcpy(sdp->path_name, filesystem);
-	check_for_gfs2(sdp);
-	read_superblock(&sdp->sd_sb, sdp);
-	sprintf(sys_quota_sync, "%s%s%s", 
-		"/sys/fs/gfs2/", sdp->sd_sb.sb_locktable, "/quota_sync");
-	
-	fd = open(sys_quota_sync, O_WRONLY);
-	if (fd < 0)
-		die("can't open file %s: %s\n", sys_quota_sync, strerror(errno));
-	
-	if (write(fd,(void*)"1", 1) != 1)
-		die("failed to write to %s: %s\n", 
-		    sys_quota_sync, strerror(errno));
-	
-	close(fd);
+	fsname = mp2fsname(filesystem);
+	set_sysfs(fsname, "quota_sync", "1");
 }
 
 /**
@@ -540,14 +792,14 @@ do_sync(struct gfs2_sbd *sdp, commandline_t *comline)
 static void
 do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 {
-	int fd, fd1;
+	int fd;
 	uint64_t offset;
 	uint64_t new_value;
-	int error;
+	int error, adj_flag = 0;;
 	char quota_file[BUF_SIZE];
-	char sys_q_refresh[BUF_SIZE];
 	char id_str[16];
 	struct stat stat_buf;
+	char *fs;
 	
 	if (!*comline->filesystem)
 		die("need a filesystem to work on\n");
@@ -564,7 +816,7 @@ do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 	strcpy(quota_file, sdp->metafs_path);
 	strcat(quota_file, "/quota");
 
-	fd = open(quota_file, O_WRONLY);
+	fd = open(quota_file, O_RDWR);
 	if (fd < 0) {
 		close(sdp->metafs_fd);
 		cleanup_metafs(sdp);
@@ -572,6 +824,11 @@ do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 		    strerror(errno));
 	}
 	
+	if (is_valid_quota_list(fd))
+		adj_flag = 1;
+	else
+		print_quota_list_warning();
+
 	switch (comline->id_type) {
 	case GQ_ID_USER:
 		offset = (2 * (uint64_t)comline->id) * sizeof(struct gfs2_quota);
@@ -684,28 +941,13 @@ do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 		}
 	}
 
-	/* Write the id to sysfs quota refresh file to refresh gfs quotas */
-	sprintf(sys_q_refresh, "%s%s%s", "/sys/fs/gfs2/",
-		sdp->sd_sb.sb_locktable, 
-		comline->id_type == GQ_ID_USER ? "/quota_refresh_user" : 
-		"/quota_refresh_group");
-	
-	fd1 = open(sys_q_refresh, O_WRONLY);
-	if (fd1 < 0) {
-		fprintf(stderr, "can't open file %s: %s\n", sys_q_refresh, 
-			strerror(errno));
-		goto out;
-	}
-
+	fs = mp2fsname(comline->filesystem);
 	sprintf(id_str, "%d", comline->id);
+	set_sysfs(fs, comline->id_type == GQ_ID_USER ?
+		  "quota_refresh_user" : "quota_refresh_group", id_str);
 	
-	if (write(fd1,(void*)id_str, strlen(id_str)) != strlen(id_str)) {
-		close(fd1);
-		fprintf(stderr, "failed to write to %s: %s\n", 
-			sys_q_refresh, strerror(errno));
-		goto out;
-	}
-	close(fd1);
+	if (adj_flag)
+		adjust_quota_list(fd, comline);
 out:
 	close(fd);
 	close(sdp->metafs_fd);
@@ -765,6 +1007,9 @@ main(int argc, char *argv[])
 		do_quota_init(sdp, &comline);
 		break;
 
+	case GQ_OP_RESET:
+		do_reset(sdp, &comline);
+		break;
 	default:
 		if (!comline.id_type) {
 			comline.id_type = GQ_ID_USER;
