@@ -1,15 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #ifndef __extern_req_h__
 #define __extern_req_h__
 
@@ -41,6 +29,7 @@ typedef struct import_info_s import_info_t;
 #define EXTERN_KILL_GSERV_REQ   5
 #define EXTERN_LOGIN_REQ        6
 #define EXTERN_NODENAME_REQ     7
+#define EXTERN_UID_REQ          8
 
 #define EXTERN_SUCCESS_REPLY    0
 /* FIXME -- is this used */
